@@ -51,9 +51,11 @@ def _normalize_line(raw_line: str) -> Tuple[str, str]:
     return (line, raw_line)
 
 
-def import_type(line: str) -> Optional[str]:
+def import_type(line: str, config: Config = DEFAULT_CONFIG) -> Optional[str]:
     """If the current line is an import line it will return its type (from or straight)"""
-    if "isort:skip" in line or "isort: skip" in line or "NOQA" in line:
+    if config.honor_noqa and line.lower().rstrip().endswith("noqa"):
+        return None
+    elif "isort:skip" in line or "isort: skip" in line:
         return None
     elif line.startswith(("import ", "cimport ")):
         return "straight"
@@ -78,7 +80,11 @@ def _strip_syntax(import_string: str) -> str:
 
 
 def skip_line(
-    line: str, in_quote: str, index: int, section_comments: List[str]
+    line: str,
+    in_quote: str,
+    index: int,
+    section_comments: Tuple[str, ...],
+    needs_import: bool = True,
 ) -> Tuple[bool, str]:
     """Determine if a given line should be skipped.
 
@@ -107,7 +113,7 @@ def skip_line(
                 break
             char_index += 1
 
-    if ";" in line:
+    if ";" in line and needs_import:
         for part in (part.strip() for part in line.split(";")):
             if (
                 part
@@ -125,14 +131,13 @@ class ParsedContent(NamedTuple):
     import_index: int
     place_imports: Dict[str, List[str]]
     import_placements: Dict[str, str]
-    as_map: Dict[str, List[str]]
+    as_map: Dict[str, Dict[str, List[str]]]
     imports: Dict[str, Dict[str, Any]]
     categorized_comments: "CommentsDict"
     change_count: int
     original_line_count: int
     line_separator: str
     sections: Any
-    section_comments: List[str]
 
 
 def file_contents(contents: str, config: Config = DEFAULT_CONFIG) -> ParsedContent:
@@ -141,7 +146,6 @@ def file_contents(contents: str, config: Config = DEFAULT_CONFIG) -> ParsedConte
     in_lines = contents.split(line_separator)
     out_lines = []
     original_line_count = len(in_lines)
-    section_comments = [f"# {heading}" for heading in config.import_headings.values()]
     if config.old_finders:
         finder = FindersManager(config=config).find
     else:
@@ -151,7 +155,10 @@ def file_contents(contents: str, config: Config = DEFAULT_CONFIG) -> ParsedConte
 
     place_imports: Dict[str, List[str]] = {}
     import_placements: Dict[str, str] = {}
-    as_map: Dict[str, List[str]] = defaultdict(list)
+    as_map: Dict[str, Dict[str, List[str]]] = {
+        "straight": defaultdict(list),
+        "from": defaultdict(list),
+    }
     imports: OrderedDict[str, Dict[str, Any]] = OrderedDict()
     for section in chain(config.sections, config.forced_separate):
         imports[section] = {"straight": OrderedDict(), "from": OrderedDict()}
@@ -170,10 +177,10 @@ def file_contents(contents: str, config: Config = DEFAULT_CONFIG) -> ParsedConte
         index += 1
         statement_index = index
         (skipping_line, in_quote) = skip_line(
-            line, in_quote=in_quote, index=index, section_comments=section_comments
+            line, in_quote=in_quote, index=index, section_comments=config.section_comments
         )
 
-        if line in section_comments and not skipping_line:
+        if line in config.section_comments and not skipping_line:
             if import_index == -1:
                 import_index = index - 1
             continue
@@ -195,7 +202,7 @@ def file_contents(contents: str, config: Config = DEFAULT_CONFIG) -> ParsedConte
             (line.strip() for line in line.split(";")) if ";" in line else (line,)  # type: ignore
         ):
             line, raw_line = _normalize_line(line)
-            type_of_import = import_type(line) or ""
+            type_of_import = import_type(line, config) or ""
             if not type_of_import:
                 out_lines.append(raw_line)
                 continue
@@ -284,7 +291,11 @@ def file_contents(contents: str, config: Config = DEFAULT_CONFIG) -> ParsedConte
 
             if type_of_import == "from":
                 cimports: bool
-                import_string = import_string.replace("import(", "import (")
+                import_string = (
+                    import_string.replace("import(", "import (")
+                    .replace("\\", " ")
+                    .replace("\n", " ")
+                )
                 if " cimport " in import_string:
                     parts = import_string.split(" cimport ")
                     cimports = True
@@ -308,19 +319,20 @@ def file_contents(contents: str, config: Config = DEFAULT_CONFIG) -> ParsedConte
                 while "as" in just_imports:
                     as_index = just_imports.index("as")
                     if type_of_import == "from":
-                        module = just_imports[0] + "." + just_imports[as_index - 1]
+                        nested_module = just_imports[as_index - 1]
+                        module = just_imports[0] + "." + nested_module
                         as_name = just_imports[as_index + 1]
-                        if as_name not in as_map[module]:
-                            as_map[module].append(as_name)
+                        if nested_module == as_name and config.remove_redundant_aliases:
+                            pass
+                        elif as_name not in as_map["from"][module]:
+                            as_map["from"][module].append(as_name)
                     else:
                         module = just_imports[as_index - 1]
                         as_name = just_imports[as_index + 1]
-                        if as_name not in as_map[module]:
-                            as_map[module].append(as_name)
-                        if "." in module:
-                            type_of_import = "from"
-                            just_imports[:as_index] = module.rsplit(".", 1)
-                            as_index = just_imports.index("as")
+                        if module == as_name and config.remove_redundant_aliases:
+                            pass
+                        elif as_name not in as_map["straight"][module]:
+                            as_map["straight"][module].append(as_name)
                     if not config.combine_as_imports:
                         categorized_comments["straight"][module] = comments
                         comments = []
@@ -433,5 +445,4 @@ def file_contents(contents: str, config: Config = DEFAULT_CONFIG) -> ParsedConte
         original_line_count=original_line_count,
         line_separator=line_separator,
         sections=config.sections,
-        section_comments=section_comments,
     )
